@@ -1,12 +1,12 @@
 package Catalyst::Controller::DBIC::API;
-$Catalyst::Controller::DBIC::API::VERSION = '2.006002';
+$Catalyst::Controller::DBIC::API::VERSION = '2.007001';
 #ABSTRACT: Provides a DBIx::Class web service automagically
 use Moose;
 BEGIN { extends 'Catalyst::Controller'; }
 
 use CGI::Expand ();
 use DBIx::Class::ResultClass::HashRefInflator;
-use JSON ();
+use JSON::MaybeXS ();
 use Test::Deep::NoTest('eq_deeply');
 use MooseX::Types::Moose(':all');
 use Moose::Util;
@@ -17,14 +17,14 @@ use namespace::autoclean;
 
 has '_json' => (
     is         => 'ro',
-    isa        => 'JSON',
+    isa        => JSON::MaybeXS::JSON(),
     lazy_build => 1,
 );
 
 sub _build__json {
 
     # no ->utf8 here because the request params get decoded by Catalyst
-    return JSON->new;
+    return JSON::MaybeXS->new;
 }
 
 with 'Catalyst::Controller::DBIC::API::StoredResultSource',
@@ -74,6 +74,7 @@ sub deserialize : Chained('setup') : CaptureArgs(0) : PathPart('') :
 
             # find out if CGI::Expand was involved
             if ( ref $req_params->{$param} eq 'HASH' ) {
+                $c->log->debug("Param '$param' decoded using CGI::Expand");
                 for my $key ( keys %{ $req_params->{$param} } ) {
 
                     # copy the value because JSON::XS will alter it
@@ -91,6 +92,7 @@ sub deserialize : Chained('setup') : CaptureArgs(0) : PathPart('') :
                 }
             }
             else {
+                $c->log->debug("Param '$param' decoded using JSON");
                 try {
                     my $value        = $req_params->{$param};
                     my $deserialized = $self->_json->decode($value);
@@ -214,7 +216,9 @@ sub object_lookup {
     my ( $self, $c, $id ) = @_;
 
     die 'No valid ID provided for look up' unless defined $id and length $id;
-    my $object = $c->req->current_result_set->find($id);
+    #my $object = $c->req->current_result_set->find($id);
+    # TODO: limit to configured (select) columns
+    my $object = $c->req->current_result_set->find($id, { %{$c->req->search_attributes}{qw(select as)} });
     die "No object found for id '$id'" unless defined $object;
     return $object;
 }
@@ -388,12 +392,17 @@ sub validate_object {
         }
     );
 
+    use Data::Dumper::Concise;
+    $c->log->debug('requires: ' . Dumper(\%requires_map) . 'allows: ' . Dumper(\%allows_map));
+
     foreach my $key ( keys %allows_map ) {
+        $c->log->debug("checking $key");
 
         # check value defined if key required
         my $allowed_fields = $allows_map{$key};
 
         if ( ref $allowed_fields ) {
+            $c->log->debug("$key is a relationship");
             my $related_source = $object->result_source->related_source($key);
             my $related_params = $params->{$key};
             my %allowed_related_map = map { $_ => 1 } @$allowed_fields;
@@ -409,14 +418,17 @@ sub validate_object {
                     )
                     )
                 {
+                    $c->log->debug("$key: $related_col");
                     $values{$key}{$related_col} = $related_col_value;
                 }
             }
         }
         else {
+            $c->log->debug("$key is a column");
             my $value = $params->{$key};
 
             if ( $requires_map{$key} ) {
+                $c->log->debug("$key is required");
                 unless ( defined($value) ) {
 
                     # if not defined look for default
@@ -429,7 +441,7 @@ sub validate_object {
             }
 
             # check for multiple values
-            if ( ref($value) && !( reftype($value) eq reftype(JSON::true) ) )
+            if ( ref($value) && !( reftype($value) eq reftype(JSON::MaybeXS::true) ) )
             {
                 require Data::Dumper;
                 die
@@ -497,13 +509,15 @@ sub update_object_from_params {
 
     foreach my $key ( keys %$params ) {
         my $value = $params->{$key};
-        if ( ref($value) && !( reftype($value) eq reftype(JSON::true) ) ) {
+        if ( ref($value) && !( reftype($value) eq reftype(JSON::MaybeXS::true) ) ) {
+            $c->log->debug("$key is a relationship");
             $self->update_object_relation( $c, $object,
                 delete $params->{$key}, $key );
         }
 
         # accessor = colname
         elsif ( $object->can($key) ) {
+            $c->log->debug("$key is an accessor");
             $object->$key($value);
         }
 
@@ -511,6 +525,7 @@ sub update_object_from_params {
         else {
             my $accessor =
                 $object->result_source->column_info($key)->{accessor};
+            $c->log->debug("$key with accessor $accessor");
             $object->$accessor($value);
         }
     }
@@ -522,11 +537,12 @@ sub update_object_from_params {
 sub update_object_relation {
     my ( $self, $c, $object, $related_params, $relation ) = @_;
     my $row = $object->find_related( $relation, {}, {} );
+    $c->log->debug("in update_object_relation");
 
     if ($row) {
         foreach my $key ( keys %$related_params ) {
             my $value = $related_params->{$key};
-            if ( ref($value) && !( reftype($value) eq reftype(JSON::true) ) )
+            if ( ref($value) && !( reftype($value) eq reftype(JSON::MaybeXS::true) ) )
             {
                 $self->update_object_relation( $c, $row,
                     delete $related_params->{$key}, $key );
@@ -547,24 +563,26 @@ sub update_object_relation {
         $row->update();
     }
     else {
+        $c->log->debug("create_related '$relation' called");
         $object->create_related( $relation, $related_params );
     }
 }
 
 
 sub insert_object_from_params {
-
-    #my ($self, $c, $object, $params) = @_;
-    my ( $self, undef, $object, $params ) = @_;
+    my ($self, $c, $object, $params) = @_;
+    #my ( $self, undef, $object, $params ) = @_;
 
     my %rels;
     while ( my ( $key, $value ) = each %{$params} ) {
-        if ( ref($value) && !( reftype($value) eq reftype(JSON::true) ) ) {
+        if ( ref($value) && !( reftype($value) eq reftype(JSON::MaybeXS::true) ) ) {
+            $c->log->debug("$key is a relationship");
             $rels{$key} = $value;
         }
 
         # accessor = colname
         elsif ( $object->can($key) ) {
+            $c->log->debug("$key is an accessor");
             $object->$key($value);
         }
 
@@ -572,6 +590,7 @@ sub insert_object_from_params {
         else {
             my $accessor =
                 $object->result_source->column_info($key)->{accessor};
+            $c->log->debug("$key with accessor $accessor");
             $object->$accessor($value);
         }
     }
@@ -615,8 +634,11 @@ sub end : Private {
 
     if ( $c->res->status == 200 ) {
         $c->stash->{ $self->stash_key }->{success} =
-            $self->use_json_boolean ? JSON::true : 'true';
-        if ( $self->return_object && $c->req->has_objects ) {
+            $self->use_json_boolean ? JSON::MaybeXS::true : 'true';
+        if ( $self->return_object
+            && $c->req->has_objects
+            && ! exists $c->stash->{ $self->stash_key }->{ $self->data_root } ) {
+            $c->log->debug('adding object data to response because return_object is enabled');
             my $returned_objects = [];
             push( @$returned_objects, $self->each_object_inflate( $c, $_ ) )
                 for map { $_->[0] } $c->req->all_objects;
@@ -625,10 +647,13 @@ sub end : Private {
                 ? $returned_objects
                 : $returned_objects->[0];
         }
+        else {
+            $c->log->debug('not adding object data to response because return_object is disabled');
+        }
     }
     else {
         $c->stash->{ $self->stash_key }->{success} =
-            $self->use_json_boolean ? JSON::false : 'false';
+            $self->use_json_boolean ? JSON::MaybeXS::false : 'false';
         $c->stash->{ $self->stash_key }->{messages} = $self->get_errors($c)
             if $self->has_errors($c);
 
@@ -697,7 +722,7 @@ Catalyst::Controller::DBIC::API - Provides a DBIx::Class web service automagical
 
 =head1 VERSION
 
-version 2.006002
+version 2.007001
 
 =head1 SYNOPSIS
 
@@ -728,7 +753,8 @@ version 2.006002
       ],
       ordered_by       => ['age'],
       search_exposes   => ['age', 'nickname', { cds => ['title', 'year'] }],
-      data_root        => 'data',
+      data_root        => 'list',
+      item_root        => 'data',
       use_json_boolean => 1,
       return_object    => 1,
       );
@@ -817,16 +843,23 @@ Controls where in stash request_data should be stored, and defaults to 'response
 
 =head3 data_root
 
-By default, the response data is serialized into
+By default, the response data of multiple item actions is serialized into
 $c->stash->{$self->stash_key}->{$self->data_root} and data_root defaults to
 'list' to preserve backwards compatibility. This is now configuable to meet
 the needs of the consuming client.
 
+=head3 item_root
+
+By default, the response data of single item actions is serialized into
+$c->stash->{$self->stash_key}->{$self->item_root} and item_root default to
+'data'.
+
 =head3 use_json_boolean
 
 By default, the response success status is set to a string value of "true" or
-"false". If this attribute is true, JSON's true() and false() will be used
-instead. Note, this does not effect other internal processing of boolean values.
+"false". If this attribute is true, JSON::MaybeXS's true() and false() will be
+used instead. Note, this does not effect other internal processing of boolean
+values.
 
 =head3 count_arg, page_arg, select_arg, search_arg, grouped_by_arg, ordered_by_arg, prefetch_arg, as_arg, total_entries_arg
 
@@ -1296,9 +1329,10 @@ status quo. The internals were revamped to use more modern tools such as Moose
 and its role system to refactor functionality out into self-contained roles.
 
 To this end, internally, this module now understands JSON boolean values (as
-represented by the JSON module) and will Do The Right Thing in handling those
-values. This means you can have ColumnInflators installed that can covert
-between JSON booleans and whatever your database wants for boolean values.
+represented by the JSON::MaybeXS module) and will Do The Right Thing in
+handling those values. This means you can have ColumnInflators installed that
+can covert between JSON booleans and whatever your database wants for boolean
+values.
 
 Validation for various *_allows or *_exposes is now accomplished via
 Data::DPath::Validator with a lightly simplified, via a subclass of
@@ -1352,7 +1386,7 @@ Samuel Kaufman <sam@socialflow.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Luke Saunders, Nicholas Perez, Alexander Hartmaier, et al..
+This software is copyright (c) 2018 by Luke Saunders, Nicholas Perez, Alexander Hartmaier, et al.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
