@@ -11,7 +11,7 @@ __PACKAGE__->config
       create_allows => ['cd', 'title', 'position' ],
       update_allows => ['title', 'position', { cd => ['*'] }],
       grouped_by => ['position'],
-      select => ['position'],
+      select => ['trackid', 'cd', 'position'],
       ordered_by => ['position'],
 	  search_exposes => ['title']
       );
